@@ -60,6 +60,8 @@ int main (int argc, char **argv) {
     char current_pretty_name[255];
     int id;
     gboolean isportrait=FALSE;
+    osso_return_t result;
+    GError* error;
     
     appdata = &appdata_obj;
     
@@ -81,17 +83,29 @@ int main (int argc, char **argv) {
 #endif
 
     printf("About to do osso_initialize()\n");
-    appdata->osso = osso_initialize("org.maemo.mbarcode", "0.0.2", FALSE, NULL);
+    appdata->osso = osso_initialize("org.maemo.mbarcode", "0.0.5", FALSE, NULL);
     if(!appdata->osso){
         printf("OSSO initialisation error\n");
         close_db();
         exit(1);
     }
-        
-#ifdef N900
+    
+    result = osso_rpc_set_cb_f(appdata->osso,
+                               "org.maemo.mbarcode",
+                               "/org/maemo/mbarcode",
+                               "org.maemo.mbarcode",
+                               dbus_req_handler, NULL);
+    
+    if (result != OSSO_OK) {
+        g_print("Error setting D-BUS callback (%d)\n", result);
+        return OSSO_ERROR;
+    }
+    
     printf("About to do osso_get_sys_dbus_connection()\n");
-    appdata->con = osso_get_sys_dbus_connection(appdata->osso);
-#endif
+    
+    // http://wiki.maemo.org/Documentation/Maemo_5_Developer_Guide/Using_Data_Sharing/SendVia_Functionality
+    //appdata->con = osso_get_sys_dbus_connection(appdata->osso);
+    appdata->con = dbus_g_bus_get(DBUS_BUS_SYSTEM, &error);
 
     /* Initialize and create the GUI */
     initialize_gui(&appdata->program, &appdata->window, &argc, &argv, "mBarcode");
@@ -165,7 +179,44 @@ int main (int argc, char **argv) {
     return 0;
 }
 
+/*
+ Call me with e.g.
+ 
+ osso_return_t osso_rpc_run(osso_context_t *osso,
+        "org.maemo.mbarcode",
+        "/org/maemo/mbarcode",
+        "org.maemo.mbarcode",
+		"DoScan",
+		osso_rpc_t *  	retval,
+		DBUS_TYPE_INVALID) 		
 
+*/
+
+/*
+osso_application_top  	(osso_context_t *osso,
+		"mbarcode",
+		NULL	 
+	) 	
+*/
+
+/* Callback for normal D-BUS messages */
+gint dbus_req_handler(const gchar * interface, const gchar * method,
+                      GArray * arguments, gpointer data,
+                      osso_rpc_t * retval)
+{
+    osso_system_note_infoprint(appdata->osso, method, retval);
+    osso_rpc_free_val(retval);
+    
+    /*
+    // so can we now wait until the user has scanned a barcode, and return that value here?
+    // can I now set something in retval....?
+    // let's give it a go
+    retval->type = DBUS_TYPE_STRING;
+    retval->s = g_strdup(barcode);
+    */
+    
+    return OSSO_OK;
+}
 
 void notebook_page_change(GtkNotebook *notebook, GtkNotebookPage *page, guint page_num, gpointer user_data) {
     printf("notebook_page_change\n");
