@@ -34,6 +34,12 @@
 //int main(int argc, char **argv);
 void shutdown();
 
+
+gint dbus_req_handler(const gchar * interface, const gchar * method,
+                      GArray * arguments, gpointer data,
+                      osso_rpc_t * retval);
+
+
 /*
 void number_to_string(int *number, char *result);
 gboolean compare_numbers (int *number1, int *number2);
