@@ -1711,6 +1711,8 @@ int handle_barcode_data(BarcodeData *bcdata) //(char *data, int barcode_type)
     char *data = bcdata->message_output;
     int barcode_type = bcdata->message_type;
     
+    signal_barcode_payload(data, bcdata->message_type);
+    
     // decide what sort of data this is, and decide what we want to do with it
     fprintf(stdout, "Entered handle_barcode_data()\n");
     if (barcode_type == BARCODE_TYPE_DMTX){
@@ -1813,6 +1815,23 @@ int parse_mixed_barcode_data(char *data)
     }
 
 
+
+
+    // Let's try something Maemo specific - link to an app. E.g. and install file in a barcode
+    // we should probably parse the file and let the user know what it's for
+    // then we can also save the install file somewhere, then pass it onto app mngr
+    // so, what do we name the file? Use something from the contents (so it's understandable)
+    // or give it a unique (and not very useful) name?
+    if (strncasecmp(data, "[install]", 9)==0){
+        printf("Maemo install file\n");
+        banner = hildon_banner_show_information (appdata->window, NULL, "Maemo install file");
+        hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
+        
+        openInstallFile(data);        
+        return TRUE;
+    }
+
+
     // see if this contains an URL
     // we should check for this last, as there may be an URL in e.g. a VCard
     
@@ -1993,25 +2012,9 @@ int parse_mixed_barcode_data(char *data)
     }
 
 
-/*
-    // Let's try something Maemo specific - link to an app. E.g. and install file in a barcode
-    // we should probably parse the file and let the user know what it's for
-    // then we can also save the install file somewhere, then pass it onto app mngr
-    // so, what do we name the file? Use something from the contents (so it's understandable)
-    // or give it a unique (and not very useful) name?
-    if (strncasecmp(data, "[install]", 9)==0){ 
-        FILE *install_file;
-        char filename;
-        
-        install_file = fopen(const char *path, const char *mode);
-        printf("Maemo install file\n");
-        banner = hildon_banner_show_information (appdata->window, NULL, "Maemo install file");
-        hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
-        
-        openGeneric(filename, "Open install file: %s?");
-        return TRUE;
-    }
-*/
+
+
+
 
     
     // iCal
@@ -2233,11 +2236,223 @@ int handle_EAN8(char *data)
 
 
 
+
+// ===============================================================================================
+// libosso notifiers for our different payload types
+// ===============================================================================================
+
+void signal_barcode_payload(char* barcode, int type)
+{
+    // send out a generic "We have a barcode" message
+    //gchar *payload = g_strdup(barcode);
+    DBusMessage* msg;
+    DBusMessageIter args; 
+    dbus_uint32_t serial;
+
+    msg = dbus_message_new_signal("/org/maemo/mbarcode", // object name of the signal
+         "org.maemo.mbarcode", // interface name of the signal
+         "BarcodeDecoded"); // name of the signal
+
+    if (msg == NULL) 
+    { 
+        fprintf(stderr, "Message Null\n"); 
+        return; 
+    }
+       
+    // append arguments onto signal
+    dbus_message_iter_init_append(msg, &args);
+    if (!dbus_message_iter_append_basic(&args, DBUS_TYPE_STRING, &barcode)) { 
+        fprintf(stderr, "Out Of Memory!\n"); 
+        return;
+    }
+    if (!dbus_message_iter_append_basic(&args, DBUS_TYPE_INT32, &type)) { 
+        fprintf(stderr, "Out Of Memory!\n"); 
+        return;
+    }
+
+    // send the message and flush the connection
+    if (!dbus_connection_send(appdata->con, msg, &serial)) { 
+        fprintf(stderr, "Out Of Memory!\n"); 
+        return;
+    }
+    dbus_connection_flush(appdata->con); // not sure if this is needed or a good idea
+   
+    // free the message 
+    dbus_message_unref(msg);
+
+    //g_free(payload);    
+}
+
+
+
 // ===============================================================================================
 // Handlers for our different payload types
 // ===============================================================================================
 
 
+void openInstallFile(char *data1)
+{
+    FILE *install_file;
+    gchar *data = g_strdup(data1);
+    char *filename=NULL;
+    char *p, *pp, *s, *e;
+    
+    FILE *fid;
+    
+    fid = fopen("/home/user/mbarcode.debug", "w+");
+    fprintf(fid,"Start\n");
+    fflush(fid);
+    
+    // decide on a file name for our install file
+    
+    // first see if this is an install file for a package or for a repo only
+    p = strstr(data, "package"); // see if we find the keyword "package"
+    if(p){
+        fprintf(fid,"First loop found text\n");
+        fflush(fid);
+        p = strstr(p, "="); // if so, look for the "="
+        if(p){
+            p++; // skip past the =
+            // find the end of the line, so we can work out the name (it might have spaces)
+            s = strstr(p,"\r");
+            if(!s)
+                s = strstr(p,"\n");
+            
+            if(!s)
+                pp = g_strdup(p); // no line breaks found apparently
+            else
+                pp = g_strndup(p, (int)(s-p+1)); // so we don't get the line breaks
+            
+        
+            // assume we've now found the package name between p and the end of the string
+            // get rid of white space on both ends
+            fprintf(fid,"First loop about to strip\n");
+            fflush(fid);
+            pp = g_strstrip(pp);
+            
+            fprintf(fid,"First loop about to remove spaces\n");
+            fflush(fid);
+            // remove any spaces in the name
+            e = strstr(pp," "); 
+            while(e){
+                e[0] = '_';
+                e = strstr(pp," ");
+            }
+            
+            fprintf(fid,"First loop generating filename: %s\n", pp);
+            fflush(fid);
+            //filename = g_strdup_printf("%s/%s.install", getenv ("MYDOCSDIR"), pp); // assuming the env var exists here, no checks made
+            filename = g_strdup_printf("/home/user/MyDocs/%s.install", pp);
+            g_free(pp);
+
+        }else{ // something wrong here
+            fprintf(fid,"First loop no =\n");
+            fflush(fid);
+        }
+        
+    }
+
+    
+    fprintf(fid,"Left first loop\n");
+    fflush(fid);
+    
+    if(!filename){
+        // it might just have been a repo, so try to extract this name now
+        p = strstr(s, "repo_name"); // see if we find the keyword "package"
+        if(p){
+            fprintf(fid,"Second loop found text\n");
+            fflush(fid);
+            p = strstr(p, "="); // if so, look for the "="
+            if(p){
+                p++; // skip past the =
+                // find the end of the line, so we can work out the name (it might have spaces)
+                s = strstr(p,"\r");
+                if(!s)
+                    s = strstr(p,"\n");
+                
+                if(!s)
+                    pp = g_strdup(p); // no line breaks found apparently
+                else
+                    pp = g_strndup(p, (int)(s-p+1)); // so we don't get the line breaks
+            
+                // assume we've now found the package name between p and the end of the string
+                // get rid of white space on both ends
+                fprintf(fid,"First loop about to strip\n");
+                fflush(fid);
+                pp = g_strstrip(pp);
+
+                // remove any spaces in the name
+                fprintf(fid,"Second loop about to remove spaces\n");
+                fflush(fid);
+                e = strstr(pp," "); 
+                while(e){
+                    e[0] = '_';
+                    e = strstr(pp," ");
+                }
+
+                fprintf(fid,"Second loop generating filename: %s\n", pp);
+                fflush(fid);
+                //filename = g_strdup_printf("%s/%s.install", getenv ("MYDOCSDIR"), pp); // assuming the env var exists here, no checks made
+                filename = g_strdup_printf("/home/user/MyDocs/%s.install", pp);
+                g_free(pp);
+            
+            }else{ // something wrong here
+                fprintf(fid,"Second loop no =\n");
+                fflush(fid);
+                
+            }       
+        }            
+    }
+    
+    fprintf(fid,"Left second loop\n");
+    fflush(fid);
+    
+    
+    if(!filename){
+        // we'll just have to generate a filename now it seems
+        //filename = g_strdup_printf("%s//%s.install", getenv ("MYDOCSDIR"), tmpnam(NULL));
+        filename = g_strdup_printf("/home/user/MyDocs/%s.install", tmpnam(NULL));
+        fflush(fid);
+        fprintf(fid,"Generating random filename\n");
+    }
+    
+    /*
+    // see if it already exists
+    install_file = fopen(filename, "r");
+    if(install_file){
+        // it already exists!
+        fclose(install_file);
+    }
+    */
+    
+
+    if((install_file = fopen(filename, "w+")) == NULL){
+        fprintf(fid,"Failed to open filename\n");
+        fflush(fid);
+        g_free(filename);
+        g_free(data);
+        return; // and give up, for want of a better way of recovering...
+    }
+    
+    fprintf(fid,"Writing data\n");
+    fflush(fid);
+    
+    fwrite(data, 1, strlen(data), install_file);
+    fclose(install_file);
+
+    fprintf(fid,"about to openGeneric()\n");
+    fflush(fid);
+    
+    openGeneric(filename, "Open install file: %s?");
+
+    fclose(fid);
+    g_free(filename);
+    g_free(data);
+
+}
+
+
+
 void openGeneric(char *uri1, char *question)
 {
     GtkWidget *dialog;
